using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Collections;
using System.IO;
using System.Text.RegularExpressions;
using System.Drawing.Drawing2D;


namespace PanoramaCam
{
    public partial class PositionForm : Form
    {
        Hashtable imageHolder = new Hashtable();
        Hashtable rectHolder = new Hashtable();

        bool mousePressed = false;
        int selectedImage = -1;
        Point currentMousePoint = Point.Empty, previousMousePoint = Point.Empty;

        int xMax = 0;
        int yMax= 0;
        Size imageSize = Size.Empty;

        BufferedGraphics graphicsBuffer = null;
        BufferedGraphicsContext currentContext;

        public Hashtable PanoramaRectHolder
        {
            //set {this.rectHolder = value;}
            get { return this.rectHolder; }
        }

        public PositionForm(string directory, int xMax, int yMax, Size size)
        {
            InitializeComponent();
            this.xMax = xMax;
            this.yMax = yMax;
            this.imageSize = size;
            LoadImage( directory);
           
            panel1.Width = xMax * size.Width;
            panel1.Height = yMax * size.Height;
           
            currentContext = BufferedGraphicsManager.Current;
            graphicsBuffer = currentContext.Allocate(panel1.CreateGraphics(), panel1.DisplayRectangle);

            UpdateGraphics();
        }

        public PositionForm(string directory, int xMax, int yMax, Size size, Hashtable rectHolder)
        {
            InitializeComponent();
            this.rectHolder = rectHolder;
            this.xMax = xMax;
            this.yMax = yMax;
            this.imageSize = size;
            LoadImage(directory);

            panel1.Width = xMax * size.Width;
            panel1.Height = yMax * size.Height;
            UpdateGraphics();
            
            currentContext = BufferedGraphicsManager.Current;
            graphicsBuffer = currentContext.Allocate(panel1.CreateGraphics(), panel1.DisplayRectangle);

            UpdateGraphics();
        }


        void LoadImage(string directory)
        {
            string[] files = Directory.GetFiles(directory);
            foreach (string file in files)
            {
                string name = Path.GetFileName(file);
                Match m = Regex.Match(name, @"(\d+)[.](jpg)$");
                if (m.Success)
                {
                    int num = int.Parse(m.Groups[1].Value);
                    if (imageHolder.Contains(num))
                    {
                        imageHolder[num] = Bitmap.FromFile(file);
                        
                    }
                    else
                    {
                        imageHolder.Add(num, Bitmap.FromFile(file));
                    }
                }
            }
        }
       
        void UpdateGraphics()
        {
            if (graphicsBuffer == null)
                return;
          

            graphicsBuffer.Graphics.Clear(Color.White);

            for (int i = 0; i < imageHolder.Count; i++)
            {
                Image image = (Image)imageHolder[i];
                Rectangle rect;

                if (rectHolder.Contains(i))
                {
                    rect = (Rectangle)rectHolder[i];
                }
                else
                {
                    //int x = (image.Width) * (i % lineMax);
                    //int y = (image.Height) * (i / lineMax);
                    int x = imageSize.Width * xMax;
                    int y = imageSize.Height * yMax;

                    rect = new Rectangle(x, y, image.Width, image.Height);
                    rectHolder.Add(i, rect);
                }
               
                graphicsBuffer.Graphics.DrawImage(image, rect);
            }
            graphicsBuffer.Render();
            graphicsBuffer.Render(panel1.CreateGraphics());
            panel1.Update();
        }


        bool MoveCurrentImage(Point current, Point previous)
        {
            if (current == Point.Empty || previous == Point.Empty)
                return false;

            if (selectedImage == -1)
            {

                for (int i = 0; i < rectHolder.Count; i++)
                {
                    Rectangle rect = (Rectangle)rectHolder[i];
                    if (rect.Contains(current.X, current.Y))
                    {
                        rect.X = (current.X - previous.X) + rect.X;
                        rect.Y = (current.Y - previous.Y) + rect.Y;
                        rectHolder[i] = rect;
                        selectedImage = i;
                        return true;
                    }
                }
            }
            else
            {
               Rectangle rect = (Rectangle)rectHolder[selectedImage];
               rect.X = (current.X - previous.X) + rect.X;
               rect.Y = (current.Y - previous.Y) + rect.Y;
               rectHolder[selectedImage] = rect;
                return true;
            }
            return false;
        }


        private void panel1_MouseDown(object sender, MouseEventArgs e)
        {
            mousePressed = true;
            currentMousePoint = e.Location;
            UpdateGraphics();
            previousMousePoint = e.Location;
        }

        private void panel1_MouseMove(object sender, MouseEventArgs e)
        {
            if (mousePressed)
            {
                currentMousePoint = e.Location;
                if (MoveCurrentImage(currentMousePoint, previousMousePoint))
                {
                    UpdateGraphics();
                }
                previousMousePoint = e.Location;
            }
        }

        private void panel1_MouseUp(object sender, MouseEventArgs e)
        {
            mousePressed = false;
            selectedImage = -1;
            UpdateGraphics();
            previousMousePoint = Point.Empty;
            currentMousePoint = e.Location;
        }


        private void panel1_MouseEnter(object sender, EventArgs e)
        {
            UpdateGraphics();
        }

        private void PositionForm_Scroll(object sender, ScrollEventArgs e)
        {
            UpdateGraphics();
        }

        private void panel1_MouseLeave(object sender, EventArgs e)
        {
            UpdateGraphics();
        }

        private void buttonInit_Click(object sender, EventArgs e)
        {
            for (int j = 0; j < yMax; j++)
            {
                for (int i = 0; i < xMax; i++)
                {
                    int cnt = i + j * xMax;
                    Rectangle rect = (Rectangle)rectHolder[cnt];
                    rect.X = rect.Width * i;
                    rect.Y = (rect.Height) * j;
                    rectHolder[cnt] = rect;
                }
            }
            UpdateGraphics();
        }

        private void panelDummy_Scroll(object sender, ScrollEventArgs e)
        {
            UpdateGraphics();
        }

        private void PositionForm_Load(object sender, EventArgs e)
        {
            UpdateGraphics();
        }
    }
}