using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Threading;
using System.Runtime.InteropServices;

namespace PanoramaCam
{
    class QCamCtrl
    {

        #region \/萔/ϐ

        public struct PARAM_VENDOR_REQUEST
        {
            public short function;
            public int transferFlags;
            public byte reservedBits;
            public byte request;
            public short value;
            public short index;
            public byte dummy1;
            public byte dummy2;
        }

        public struct PARAM_VENDOR_REQUEST_WRITE
        {
            public short function;
            public int transferFlags;
            public byte reservedBits;
            public byte request;
            public short value;
            public short index;
            public byte dummy1;
            public byte dummy2;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = MAX_BUFSIZE_BULK)]
            public byte[] buf;
        }

        const int IOCTL_BULKUSB_VENDORREQUEST_OUT = 0x220014; 
        const string CTL_DEVICE_NAME = @"\\.\filtCtlp";
        const int URB_FUNCTION_VENDOR_DEVICE = 0x17;
        const int SIZE_PARAM_VENDOR_REQUEST = 0x10;
        const int MAX_BUFSIZE_BULK = 1124;

        private const int COMMAND_ID_UP = 0;
        private const int COMMAND_ID_DOWN = 1;
        private const int COMMAND_ID_LEFT = 2;
        private const int COMMAND_ID_RIGHT = 3;
        private const int COMMAND_ID_HOME1 = 4;
        private const int COMMAND_ID_HOME2 = 5;

        private static byte[][] COMMAND_BYTES = new byte[][]{
                         new byte[]{ 0x00, 0x00, 0x80, 0xFF },// up
                         new byte[]{ 0x00, 0x00, 0x80, 0x00 },// down
                         new byte[]{ 0x80, 0xFF, 0x00, 0x00 },// left
                         new byte[]{ 0x80, 0x00, 0x00, 0x00 },// right
                         new byte[]{ 0x00, 0x5A, 0x00, 0x5A },// home1
                         new byte[]{ 0x80, 0xEE, 0xC0, 0xF8 } // home2
                        };//new byte[6][]

        const int COMMAND_BYTE_LENGTH = 4;
        const int DEVICE_ID_MAX = 10;

        // Win32 API function & define
        const uint GENERIC_READ = (uint)0x80000000;
        const uint GENERIC_WRITE = 0x40000000;
        const uint FILE_SHARE_READ = 0x1;
        const uint FILE_SHARE_WRITE = 0x2;
        const uint OPEN_EXISTING = 0x3;
        const int INVALID_HANDLE_VALUE = -1;

        int m_hctlUSB = -1;
        Hashtable deviceIDHolder = null;

        #endregion
    
        #region DllImport̐錾

         [DllImport("kernel32", SetLastError = true)] // , CharSet=CharSet.Unicode, ExactSpelling=true)
        static unsafe extern int DeviceIoControl(int hDevice, int dwIoControlCode,
                                                ref PARAM_VENDOR_REQUEST_WRITE lpInBuffer,
                                                int nInBufferSize, 
                                                ref object lpOutBuffer, 
                                                int nOutBufferSize,
                                                ref int lpBytesReturned, 
                                                int lpOverlapped);
                                    // ref PARAM_VENDOR_REQUEST_WRITE lpInBuffer
        // ref PARAM_VENDOR_REQUEST_WRITE

        [DllImport("kernel32", SetLastError = true)] // , CharSet=CharSet.Unicode, ExactSpelling=true)
        static extern int CreateFile(string lpFileName, uint dwDesiredAccess, uint dwShareMode,
                                uint lpSecurityAttributes, uint dwCreationDisposition,
                                uint dwFlagsAndAttributes, uint hTemplateFile);


        [DllImport("kernel32", SetLastError = true)]
        static extern int CloseHandle(int hObject);

        [DllImport("kernel32", SetLastError = true)]
        static extern int Sleep(int dwMilliseconds);

        #endregion

        public QCamCtrl()
        {
            Initialize();
        }

        ~QCamCtrl()
        {
            Dispose();
        }

        public void Initialize()
        {
            string deviceName;
            int deviceID;
            int h_ctrl = INVALID_HANDLE_VALUE;

            deviceIDHolder = new Hashtable();

            for (int i = 0; i < DEVICE_ID_MAX; i++)
            {
                deviceID = i;
                deviceName = CTL_DEVICE_NAME + deviceID.ToString();
                h_ctrl = CreateFile(deviceName, GENERIC_READ | GENERIC_WRITE,
                                    FILE_SHARE_READ | FILE_SHARE_WRITE, 0, OPEN_EXISTING, 0, 0);

                if (h_ctrl != INVALID_HANDLE_VALUE)
                {
                    deviceIDHolder.Add(deviceIDHolder.Count, deviceID);
                    CloseHandle(h_ctrl);
                }
            }

            m_hctlUSB = INVALID_HANDLE_VALUE;
            return;
        }

        public int GetDeviceID(int num)
        {
            if (deviceIDHolder.Count == 0)
            {
                return -1;
            }
            else
            {
                return (int)deviceIDHolder[num];
            }

        }

        public bool GetEnabled()
        {
            if (m_hctlUSB != INVALID_HANDLE_VALUE)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public void Dispose()
        {
            if (m_hctlUSB == INVALID_HANDLE_VALUE)
                return;

            CloseHandle(m_hctlUSB);

            m_hctlUSB = INVALID_HANDLE_VALUE;
        }

        public bool Open(int id)
        {
            string deviceName;

            if (m_hctlUSB != INVALID_HANDLE_VALUE)
            {
                return false;
            }

            deviceName = CTL_DEVICE_NAME + id.ToString();
            m_hctlUSB = CreateFile(deviceName, GENERIC_READ | GENERIC_WRITE,
                                FILE_SHARE_READ | FILE_SHARE_WRITE, 0, OPEN_EXISTING, 0, 0);


            if (m_hctlUSB != INVALID_HANDLE_VALUE)
            {
                return true;
            }
            else
            {
                return false;
            }

        }

      
        public bool SendCommandMove(QcamCommand command)
        {

            if (m_hctlUSB == INVALID_HANDLE_VALUE)
            {
                return false;
            }

            if (command.value <= 0)
            {
                return false;
            }

            try
            {
                int commandID = -1;

                switch (command.command)
                {

                    case QcamCommand.COMMAND_UP:
                       commandID = COMMAND_ID_UP;
                        break;
                    case QcamCommand.COMMAND_DOWN:
                        commandID =  COMMAND_ID_DOWN;
                        break;
                    case QcamCommand.COMMAND_LEFT:
                        commandID = COMMAND_ID_LEFT;
                        break;
                    case QcamCommand.COMMAND_RIGHT:
                        commandID = COMMAND_ID_RIGHT;
                        break;
                }

                for (int i = 0; i < command.value; i++)
                {
                    ClassVendorRequestWrite(m_hctlUSB, COMMAND_BYTES[commandID], COMMAND_BYTE_LENGTH);
                    Thread.Sleep(100);
                }
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }

       public  bool SendCommandHome()
        {
            if (m_hctlUSB == INVALID_HANDLE_VALUE)
            {
                return false;
            }
   
            try
            {
                ClassVendorRequestWrite(m_hctlUSB, COMMAND_BYTES[COMMAND_ID_HOME1], COMMAND_BYTE_LENGTH);
                Thread.Sleep(1500);
                ClassVendorRequestWrite(m_hctlUSB, COMMAND_BYTES[COMMAND_ID_HOME2], COMMAND_BYTE_LENGTH);
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }


        private int ClassVendorRequestWrite(int handle, byte[] cmd, short wLength)
        {
            int length = SIZE_PARAM_VENDOR_REQUEST + wLength;
            int result = 0, size = 0;
            PARAM_VENDOR_REQUEST_WRITE paramData = new PARAM_VENDOR_REQUEST_WRITE();
            object empty = null;

            paramData.function = URB_FUNCTION_VENDOR_DEVICE;
            paramData.transferFlags = 0;
            paramData.reservedBits = 0;
            paramData.request = 0x0D;
            paramData.value = 0x01;
            paramData.index = 0x03;
            paramData.dummy1 = 0;
            paramData.dummy2 = 0;

            paramData.buf = new byte[MAX_BUFSIZE_BULK];
            paramData.buf[0] = cmd[0];
            paramData.buf[1] = cmd[1];
            paramData.buf[2] = cmd[2];
            paramData.buf[3] = cmd[3];

            result = DeviceIoControl(handle, IOCTL_BULKUSB_VENDORREQUEST_OUT,
                                        ref paramData, length, ref empty, 0, ref size, 0);

            wLength = (short)size;

            return result;
        }
    }
}
